<?php

/**
 * ga-core.php
 * Works with GAPI to get and manage data from Google Analytics
 *
 * @author Andrew Kallemeyn <ak@ireed.net>
 */

class GoogleAnalytics {
	private $token;
	private $profileID;
	private $encryptionSalt;
	private $timePeriod;
	private $cacheLastUpdated;
	private $cachedData;

	/* PHP5 Construct */
	function __construct($profileID = false) {
		$this->encryptionSalt = md5('domenatwo');

		$gai_data 	= get_option('gai_data');
		$gai_cache	= get_option('gai_cache');

		$this->profileID 		= $profileID;
		$this->cacheData 		= array();
		$this->cacheLastUpdated = 0;

		if($gai_data) {
			if(isset($gai_data['token'])) {
				$this->token = $this->decrypt($gai_data['token']);
			}

			if($gai_cache && $profileID !== false) {
				$cachedData = $gai_cache[$profileID];

				if($cachedData) {
					$this->cachedData 		= $gai_cache;
					$this->cacheLastUpdated = $cachedData['last_updated'];
				}
			}
		}
	}
	    
    function encrypt($token) { 
        return trim(base64_encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $this->encryptionSalt, $token, MCRYPT_MODE_ECB, mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)))); 
    } 

    function decrypt($token) { 
        return trim(mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $this->encryptionSalt, base64_decode($token), MCRYPT_MODE_ECB, mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND))); 
    }
	
	/**
	 * Exchanges the given single-use token for a session
	 * token using AuthSubSessionToken, and returns the result.
	 */
	function exchange_token($token) {
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, "https://www.google.com/accounts/AuthSubSessionToken");
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_FAILONERROR, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Authorization: AuthSub token="' . $token . '"'));

		$result = curl_exec($ch);
		curl_close($ch);

		$splitStr = explode("=", $result);

		if(isset($splitStr[1])) {
			return trim($splitStr[1]);
		}else{
			return false;
		}
	}
	
	function has_token() {
		//checks to see if a token is available
		if($this->token != '') {
			return true;
		}else{
			return false;
		}
	}

	function process_token($token, $redirectURL) {
		//Exchange the given single-use token for a session token
		if ($token) {
			$sessionToken = $this->exchange_token($token);

			if($sessionToken) {
				$this->token = $sessionToken;

				//get the domains for this account
				$ga = new gapi('','', $sessionToken);
				$ga->requestAccountData();

				$arDomains = array();
				$defaultProfileId = 0;

				foreach($ga->getResults() as $result) {
					if($defaultProfileId == 0) {
						$defaultProfileId = $result->getProfileId();
					}

					$arDomains[$result->getProfileId()] = array(
						'domainName' 	=> str_replace('Google Analytics View (Profile) ', '', $result) . '',
						'webPropertyID' => $result->getWebPropertyId() //used for tracking embed code
					);
				}

				$data = array(
					'token'		=> $this->encrypt($sessionToken),
					'domains'	=> $arDomains
				);

				//store this data
				update_option('gai_data', $data);

				//redirect back to the admin
				header("Location: " . $redirectURL);
				exit;

			}else{
				if($this->_iscurlinstalled()) {
					throw new Exception("Could not get a session token - the token has probably expired, which happens if this page was reloaded from the browser. Use the back button on your browser to try again.");
				}else{
					throw new Exception("Could not get a session token - CURL is not installed on your server. To enable Google Analytics, your server needs cURL installed. Please check with your web hosting provider.");					
				}
				exit;
			}
		}
	}
	
	function destroy_token() {
		//removes the session token from the database - forces reauthentication with Google
		update_option('gai_data', '');
	}

	function update_cache() {
		$this->cachedData[$this->profileID] = $this->get_latest_data();
		update_option('gai_cache', $this->cachedData);
	}
	
	function check_cache() {
		//if the cache is more than 24 hours old, update it...

		if((time() - $this->cacheLastUpdated) > 86400) {
			$this->update_cache();
		}
	}
	
	function get_cached_data() {
		return get_option('gai_cache');
	}
	
	function get_latest_data() {
		$ga = new gapi('','', $this->token);

		$dimension = "date";
		$endTimestamp = strtotime("yesterday");
		$midTimestamp = strtotime("-" . ($this->timePeriod / 2) . " days");
		$startTimestamp = strtotime("-" . $this->timePeriod . " days");

		$ga->requestReportData($this->profileID, array($dimension), array('pageviews','visits','percentNewVisits'), array($dimension), null, date("Y-m-d", $startTimestamp), date("Y-m-d", $endTimestamp), 1, 90);

		$dataPoints = array();

		foreach($ga->getResults() as $result) {
			$dataPoints["$result"] = $result->getVisits();
		}

		$yAxisMin 	= 0;
		$yAxisMax 	= max($dataPoints);	
		$visits	  	= $ga->getVisits();
		$page_views = $ga->getPageViews();
		$new_visits = round($ga->getPercentNewVisits(), 2);

		//to get unique visits, we need to run a separate query without dimensions
		//more info: http://groups.google.com/group/google-analytics-data-export-api/browse_thread/thread/fcdda0cbb450e460
		$ga->requestReportData($this->profileID, null, array('visitors'), null, null, date("Y-m-d", $startTimestamp), date("Y-m-d", $endTimestamp), 1, 1);

		$unique_visits = $ga->getVisitors();

		$data = array(
			'chart_url' => "http://chart.apis.google.com/chart?chf=bg,s,67676700&chxl=0:|".urlencode(date("M j Y", $startTimestamp))."|".urlencode(date("M j Y", $midTimestamp))."|".urlencode(date("M j Y", $endTimestamp))."&chxp=0,0,15,30&chxr=0,1,31|1,".$yAxisMin.",".$yAxisMax."&chxs=0,__AXISCOLOR__,12.5,0,l,E0E0E0|1,__AXISCOLOR__,12.5,0,l,000000&chxt=x,y&chs=__WIDTH__x__HEIGHT__&cht=lc&chco=__LINECOLOR__,76A4FB&chds=".$yAxisMin.",".$yAxisMax.",0,96.667&chd=t1:".implode(",", $dataPoints)."|0,0&chg=50,50,1,5&chls=3|0.75&chma=10,10,10,10&chm=o,__LINECOLOR__,0,-1,8|b,__BOUNDARYFILLCOLOR__,1,0,0",
			'visits' => $visits,
			'unique_visits' => $unique_visits,
			'page_views' => $page_views,
			'new_visits_percentage' => $new_visits,
			'last_updated' => time(),
			'time_period' => $this->timePeriod
		);

		return $data;
	}

	function set_time_period($time_period) {
		$this->timePeriod = $time_period;
	}

	function clear_cache() {
		//set last_updated to 0 - cache will be refreshed on next hit to the website
		$this->cachedData[$this->profileID]['last_updated'] = 0;
		update_option('gai_cache', $this->cachedData);
	}
	
	function get_time_period() {
		return $this->timePeriod;
	}

	function _iscurlinstalled() {
		if  (in_array  ('curl', get_loaded_extensions())) {
			return true;
		}
		else{
			return false;
		}
	}
}